------------------------------------------------------------------------
-- Copyright Notice:                                                    
-- (c) Copyright 2004, National Optical Astronomy Observatory, which is 
-- operated for the National Science Foundation by the Association of   
-- Universities for Research in Astronomy under Cooperative Agreement   
-- No. AST-0132798.                                                     
--                                                                      
------------------------------------------------------------------------
-- PROJECT      :   MONSOON
-- BOARD        :   GENERIC
-- DEVICE ID    :   U22 (IR ACQ FPGA)
-- PARENT MODULE:   SEQUENCER TOP LEVEL (SEQ_FPGA)  
-- MODULE       :   Loop Begin Address Stack 
-- VERSION      :   1.1 
------------------------------------------------------------------------
-- FILE NAME    :   Loop_Addr_Stack.VHD
-- AUTHOR       :   KAVIRAJ CHOPRA
-- DATE         :   ??/??/2003
-- COMPANY      :   NOAO
-- PLATFORM     :   Xilinx ISE 4.2
------------------------------------------------------------------------
-- DESCRIPTION  :   
-- The module is a simple 12 bit x 64 word RAM. Functions as a Program
-- Counnter Stack to store the Loop begin Address inorder to facilitate 
-- the nested LOOP structures.  
------------------------------------------------------------------------
-- IMPLEMENTATION NOTES :
-- The module will be implemented as a distributed RAM, 
-- The current depth of  the RAM is 16 words however it can be furhter
-- increased as and when needed.
------------------------------------------------------------------------
-- REVISION HISTORY :
-- DATE	      AUTHOR  VERSION  MODIFICATIONS
-- 02/22/2003  KAVI     1.0    ORIGINAL VERSION
-- ??/??/2003  KAVI     1.1    INCREASED STACK DEPTH TO 64
--
------------------------------------------------------------------------
-- PORT DEFINITIONS 	:
-- LASPtr : Memory address 4 bits :- 16 words
-- LASWriteData : Write port : 12 bit 
-- LASWrite : Write Strobe
-- LASRdData : Read Port : 12 bit
-- SysClk : System Clock 40 MHz.
------------------------------------------------------------------------


library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity Loop_Addr_Stack is
    Port ( LASPtr : in std_logic_vector(5 downto 0);
           LASWriteData : in std_logic_vector(11 downto 0);
           LASWrite : in std_logic;
           LASRdData : out std_logic_vector(11 downto 0);
		   SysClk	: in std_logic);
end Loop_Addr_Stack;

architecture Behavioral of Loop_Addr_Stack is

 type ram_type is array (63 downto 0) of std_logic_vector (11 downto 0); 
 signal RAM : ram_type; 

begin

 process (SysClk) 
 begin 
 	if rising_edge (SysClk) then  
 		if (LASWrite = '1') then 
 			RAM(conv_integer(LASPtr)) <= LASWriteData; 
 		end if; 
 	end if; 
 end process;
 
 LASRDData <= RAM(conv_integer(LASPtr)); 

end Behavioral;

